Mesa (master): r300/compiler: fix the saturate modifier when applied to TEX instructions

Marek Olšák mareko at kemper.freedesktop.org
Sat Mar 12 07:50:58 UTC 2011


Module: Mesa
Branch: master
Commit: e4707604ab46252d2e010906124775d3300f0be1
URL:    http://cgit.freedesktop.org/mesa/mesa/commit/?id=e4707604ab46252d2e010906124775d3300f0be1

Author: Marek Olšák <maraeo at gmail.com>
Date:   Sat Mar 12 06:07:24 2011 +0100

r300/compiler: fix the saturate modifier when applied to TEX instructions

This bug can only be triggered if the source texture is either signed or float.

---

 .../drivers/dri/r300/compiler/radeon_program_tex.c |    6 +++++-
 1 files changed, 5 insertions(+), 1 deletions(-)

diff --git a/src/mesa/drivers/dri/r300/compiler/radeon_program_tex.c b/src/mesa/drivers/dri/r300/compiler/radeon_program_tex.c
index 0403b8b..1cf77d9 100644
--- a/src/mesa/drivers/dri/r300/compiler/radeon_program_tex.c
+++ b/src/mesa/drivers/dri/r300/compiler/radeon_program_tex.c
@@ -425,17 +425,21 @@ int radeonTransformTEX(
 		scale_texcoords(compiler, inst, RC_STATE_R300_TEXSCALE_FACTOR);
 	}
 
 	/* Cannot write texture to output registers (all chips) or with masks (non-r500) */
 	if (inst->U.I.Opcode != RC_OPCODE_KIL &&
 		(inst->U.I.DstReg.File != RC_FILE_TEMPORARY ||
+		 inst->U.I.SaturateMode ||
 		 (!c->is_r500 && inst->U.I.DstReg.WriteMask != RC_MASK_XYZW))) {
 		struct rc_instruction * inst_mov = rc_insert_new_instruction(c, inst);
 
 		inst_mov->U.I.Opcode = RC_OPCODE_MOV;
+		inst_mov->U.I.SaturateMode = inst->U.I.SaturateMode;
 		inst_mov->U.I.DstReg = inst->U.I.DstReg;
 		inst_mov->U.I.SrcReg[0].File = RC_FILE_TEMPORARY;
 		inst_mov->U.I.SrcReg[0].Index = rc_find_free_temporary(c);
 
+		inst->U.I.SaturateMode = 0;
 		inst->U.I.DstReg.File = RC_FILE_TEMPORARY;
 		inst->U.I.DstReg.Index = inst_mov->U.I.SrcReg[0].Index;
 		inst->U.I.DstReg.WriteMask = RC_MASK_XYZW;
@@ -425,17 +425,21 @@ int radeonTransformTEX(
 	}
 
-	/* Cannot write texture to output registers (all chips) or with masks (non-r500) */
+	/* Cannot write texture to output registers or with saturate (all chips),
+	 * or with masks (non-r500). */
 	if (inst->U.I.Opcode != RC_OPCODE_KIL &&
 		(inst->U.I.DstReg.File != RC_FILE_TEMPORARY ||




More information about the mesa-commit mailing list